#!/bin/bash
	declare -r UsbDevice="/dev/sda"
	declare -r UsbPartition="/dev/sda1"
	declare -r UsbMount="/media"
	declare -r VendorXML="$UsbMount/vendor.xml"
	declare -r logfile="$UsbMount/log.txt"
	declare -r platformXML="/mnt/platform.xml"

	if [ "$UID" -ne 0 ]
	then
  		echo "Only root can run this script."
  		exit 1
	fi

	cleanup()
	# umount and delete
	{
		echo -n "Making cleanup... "
		umount "$UsbPartition"
		rmdir "$UsbMount"
		echo "OK"

		reboot
		exit 0
	}

	echo "Starting licensing..."

	. /mnt/parsexml.sh

############################################################################################################
#                                                                                                          #
#                                     Here the script starts                                               #
#                                                                                                          #
############################################################################################################

	echo "0" > /proc/splash
	clear

	cd /
	mkdir $UsbMount
	if [ $? -ne 0 ]; then
		echo "Error making directory $UsbMount. Abort."
		cleanup
	fi

	EscKey=`echo 27|awk '{printf("%c",$1)}'`
	old_tty_setting=$(stty -g)

	# wait a couple of seconf fot usb drive to recognize
	sleep 4
	Mounted="false"
	stty -icanon min 0 time 0 -echo

	while [ "$Mounted" = "false" ]
	do
		mount "$UsbPartition" "$UsbMount"
		if [ $? -ne 0 ]; then
			Mounted="false"
			#echo "Error mounting $UsbPartition."
			echo "Please plug in the USB License stick or press \"Esc\" to abort."

			# read one pressed key from terminal
			read -t 1 Key
    
			# check if the key is Esc
			if [ "$Key" = "$EscKey" ]; then
				# ask again - evaluation or permanent license?
				Answer=""
				while [ "$Answer" != "yes" -a "$Answer" != "no" ]
				do
					echo "Continue the process of getting the device permanent license? (yes/no) [yes]:"

					# read answer
					stty "$old_tty_setting"
					read Answer
					stty -icanon min 0 time 0 -echo
					if [ -z "$Answer" ]; then
						Answer="yes"
					fi
					
					if [ "$Answer" != "yes" -a "$Answer" != "no" ]; then
						echo -e "You should answer \"yes\" or \"no\".\n"
					fi
				done

				case "$Answer" in
					"no"	)
						# restore menu.lst if necessary
						MenuLstSize=`ls -l /mnt|grep menu.lst|awk '{print $5}'`
						MenuLcSize=`ls -l /mnt|grep menu.lc|awk '{print $5}'`
						if [ $MenuLstSize -eq $MenuLcSize ]; then
							if [ -f "$platformXML" ]; then
								parse "$platformXML"
								#grab OS
								i=0
								while ! [ "${xmlSTAT[i]}" = "<" -a "${xmlDATA[i]}" = "kit" ] && [ "$i" -lt "$xINDEX" ]
								do
									let "i++"
								done	
								while [ "${xmlSTAT[i]}" != "os" -a "$i" -le "$xINDEX" ]
								do
									let "i++"
								done
								if [ "$i" -lt "$xINDEX" ]; then
									OS=${xmlDATA[i]}
								else
									echo "Warning: no OS name in $platrofmXML. Can't restore menu."
									cleanup
								fi

								OS=${OS%%' '*}
								case "$OS" in
									"Linux"	)
										cp /mnt/menu.lx /mnt/menu.lst ;;
									"WinCE"	)
										cp /mnt/menu.ce /mnt/menu.lst ;;
									"WinXPe"	)
										cp /mnt/menu.xp /mnt/menu.lst ;;
								esac
							else
								cp /mnt/menu.rs /mnt/menu.lst
							fi
						fi
						cleanup
						;;
				esac
			fi
			sleep 3
		else
			Mounted="true"
		fi
	done

	stty "$old_tty_setting"

	if [ ! -f "$VendorXML" ]; then
		echo "$VendorXML is missing. Abort."
		echo `date +'|'%F'|'%T`"| $VendorXML is missing." >> $logfile
		cleanup
	fi

	if [ ! -f "$UsbMount""/tlg" ]; then
		echo "$UsbMount/tlg is missing. Abort."
		echo `date +'|'%F'|'%T`"| $UsbMount/tlg is missing." >> $logfile
		cleanup
	fi

	cd "$UsbMount"
	# launch ThinCCo License Generator
	$UsbMount/tlg.sh "$UsbDevice"
	sleep 2

	cd /
	cleanup

