#!/bin/bash
	declare -r logfile="/mnt/update.log"
	declare -r platformXML="/mnt/platform.xml"
	declare -r vendorXML="/mnt/vendor.xml"

	if [ "$UID" -ne 0 ]
	then
  		echo "Only root can run this script."
  		exit 1
	fi

	echo "Starting update..."
	echo `date +%F' '%T`" Starting update..." >> $logfile

	. /mnt/parsexml.sh

	cleanup()
	#umount and delete
	{
		ExitCmd=$1
		cd /
		echo -n "Making cleanup..."
	  	umount "$mnt_dir"
	 	rmdir "$mnt_dir"
		[ -f $ImageFile ] && rm $ImageFile
		[ -f $MD5File ] &&  rm $MD5File
		umount "/tmp"
		echo "OK"
		echo `date +%F' '%T`" Making cleanup...OK" >> $logfile

		if ! [ -z "$ExitCmd" ]; then
			$ExitCmd
		fi
		echo "Press any key to reboot..."
		read -s -n 1
		echo -e ""
		reboot
	}


	PlatformParsed="true"

	if [ -f "$platformXML" ]; then
		parse "$platformXML"

		Platform="any"
		OS="Recovery OS 1.5"
		OS_short="rs"

		i=0
		while ! [ "${xmlSTAT[i]}" = "<" -a "${xmlDATA[i]}" = "kit" ] && [ "$i" -lt "$xINDEX" ]
		do	
			let "i++"
		done

		#grab OS2 on the 2nd partition
		while [ "${xmlSTAT[i]}" != "os" -a "$i" -le "$xINDEX" ]
		do
			let "i++"
		done
		if [ "$i" -lt "$xINDEX" ]; then
			OS2=${xmlDATA[i]}
			OS2=${OS2%%' '*}
		fi

		#grab model
		i=0
		while ! [ "${xmlSTAT[i]}" = "<" -a "${xmlDATA[i]}" = "model" ] && [ "$i" -lt "$xINDEX" ]	
		do
			let "i++"
		done
		while [ "${xmlSTAT[i]}" != "val" -a "$i" -lt "$xINDEX" ]	
		do
			let "i++"
		done
		if [ "$i" -lt "$xINDEX" ]; then
			Model=${xmlDATA[i]}
		else
			echo "Warning: no model name in $platrofmXML."
			echo `date +%F' '%T`" Warning: no model name in $platrofmXML." >> $logfile
			PlatformParsed="false"
		fi

		#adjust menu.lst to avoid dead loop after cloning
 		case "$OS2" in
	 	""	) ;;
 		"WinCE"	) cp -f "/mnt/menu.ce" "/mnt/menu.lst" ;;
		"Linux"	) cp -f "/mnt/menu.lx" "/mnt/menu.lst" ;;
 		"WinXPe") cp -f "/mnt/menu.xp" "/mnt/menu.lst" ;;
 		esac
	else
		echo "Warning: $platformXML is missing."
		echo `date +%F' '%T`" Warning: $platformXML is missing." >> $logfile
		PlatformParsed="false"
	fi

	#clear parser data
	xmlSTAT=()			
	xmlDATA=()
	xINDEX=0				
	checkpoint=0				
	parserLINE=""			
	parserFLAG="ENABLED"		
	parserFILE=""			
	parserBUFF=""	


	if [ -f "$vendorXML" -a "$PlatformParsed" = "true" ]; then
		parse "$vendorXML"

		#grab ImageFile and MD5File
		i=0
		while ! [ "${xmlSTAT[i]}" = "<" -a "${xmlDATA[i]}" = "models" ] && [ "$i" -lt "$xINDEX" ]
		do
			let "i++"
		done
		while ! [ "${xmlSTAT[i]}" = "name" -a "${xmlDATA[i]}" = "$Model" ] && [ "$i" -lt "$xINDEX" ]
		do
			let "i++"
		done
		#------------------------------ We've found our model --------------------------------------------------------

		while ! [ "${xmlSTAT[i]}" = ">" -a "${xmlDATA[i]}" = "model" ] && [ "$i" -lt "$xINDEX" ]
		do
			while ! [ "${xmlSTAT[i]}" = "<" -a "${xmlDATA[i]}" = "os" ] && [ "$i" -lt "$xINDEX" ]
			do
				let "i++"
			done
			#------------- Here start attributes of some os ------------------------------------------------------
			let "i++"
			ancor=$i
			while ! [ "${xmlSTAT[i]}" = "name" ] && [ "$i" -lt "$xINDEX" ]
			do
				let "i++"
			done
			if [ "${xmlSTAT[i]}" = "name" -a "${xmlDATA[i]}" != "$OS_short" ]; then
				continue
			fi

			i=$ancor
			while ! [ "${xmlSTAT[i]}" = "platform" ] && [ "$i" -lt "$xINDEX" ]
			do
				let "i++"
			done
			if [ "${xmlSTAT[i]}" = "platform" -a "${xmlDATA[i]}" != "$Platform" ]; then
				continue
			fi

			i=$ancor
			while [ "${xmlSTAT[i]}" != "fname" -a "$i" -lt "$xINDEX" -a "${xmlSTAT[i]}" != ">" -a "${xmlDATA[i]}" != "os" ]	
			do
				let "i++"
			done
			if [ "$i" -lt "$xINDEX" -a "${xmlDATA[i]}" != "os" ]; then
				OS=${xmlDATA[i]}
			fi

			i=$ancor
			while [ "${xmlSTAT[i]}" != "file" -a "$i" -lt "$xINDEX" -a "${xmlSTAT[i]}" != ">" -a "${xmlDATA[i]}" != "os" ]	
			do
				let "i++"
			done
			if [ "$i" -lt "$xINDEX" -a "${xmlDATA[i]}" != "os" ]; then
				ImageFile=${xmlDATA[i]}
			else
				echo "Error: no image file name in $vendorXML. Abort..."
				echo `date +%F' '%T`" Error: no image file name in $vendorXML. Abort..." >> $logfile
				cleanup
				exit 1
			fi

			i=$ancor
			while [ "${xmlSTAT[i]}" != "md5" -a "$i" -lt "$xINDEX" -a "${xmlSTAT[i]}" != ">" -a "${xmlDATA[i]}" != "os" ]	
			do
				let "i++"
			done
			if [ "$i" -lt "$xINDEX" -a "${xmlDATA[i]}" != "os" ]; then
				MD5File=${xmlDATA[i]}
				DoMD5Check=0
			else
				echo "Warning: no md5 file name in $vendorXML."
				echo `date +%F' '%T`" Warning: no md5 file name in $vendorXML." >> $logfile
				DoMD5Check=1
			fi

			i=$ancor
			while [ "${xmlSTAT[i]}" != "xml" -a "$i" -lt "$xINDEX" -a "${xmlSTAT[i]}" != ">" -a "${xmlDATA[i]}" != "os" ]	
			do
				let "i++"
			done
			if [ "$i" -lt "$xINDEX" -a "${xmlDATA[i]}" != "os" ]; then
				DescriptorXML=${xmlDATA[i]}
				IsClone=0
			else
				IsClone=1
			fi

			break
		done
		#-------------------------------------------------------------------------------------------------------------

		echo "Model name: 			$Model"
		echo "OS: 				$OS"
		echo "Platform name: 			$Platform"
		echo `date +%F' '%T`" Model name: 		$Model" >> $logfile
		echo `date +%F' '%T`" OS: 			$OS" >> $logfile
		echo `date +%F' '%T`" Platform name: 		$Platform." >> $logfile

		#grab servers' url's
		declare -a Servers
		declare -i NumberOfServers=0
		i=0
		while ! [ "${xmlSTAT[i]}" = "<" -a "${xmlDATA[i]}" = "images" ] && [ "$i" -lt "$xINDEX" ]
		do
			let "i++"
		done
		while ! [ "${xmlSTAT[i]}" = "<" -a "${xmlDATA[i]}" = "server" ] && [ "$i" -lt "$xINDEX" ]
		do
			let "i++"
		done
		while [ "${xmlDATA[i]}" != "models" -a "$i" -lt "$xINDEX" ]
		do
			if [ "${xmlDATA[i]}" = "server" -a "${xmlSTAT[$((i+1))]}" = "url" ]; then
				Servers[NumberOfServers]="${xmlDATA[$((i+1))]}"
				let "NumberOfServers++"
			fi
			let "i++"
		done
	elif [ -f "$vendorXML" -a "$PlatformParsed" != "true" ]; then
		parse "$vendorXML"

		i=0;
		declare -a Options
		declare -i NumberOfOptions=0
		while ! [ "${xmlSTAT[i]}" = "<" -a "${xmlDATA[i]}" = "models" ] && [ "$i" -lt "$xINDEX" ]
		do
			let "i++"
		done
		while [ "$i" -lt "$xINDEX" ]
		do
			if [ "${xmlSTAT[i]}" = "<" -a "${xmlDATA[i]}" = "model" -a "${xmlSTAT[$((i+1))]}" = "name" ]; then
				Options[NumberOfOptions]="${xmlDATA[$((i+1))]}"
				let "NumberOfOptions++"
			fi
			let "i++"
		done

		echo "Choose your device model:"
		select opt in "${Options[@]}"; do
			Model="$opt"
			break
		done

		Platform="any"
		OS="Recovery OS 1.5"
		OS_short="rs"

		#grab ImageFile and MD5File
		i=0
		while ! [ "${xmlSTAT[i]}" = "<" -a "${xmlDATA[i]}" = "models" ] && [ "$i" -lt "$xINDEX" ]
		do
			let "i++"
		done
		while ! [ "${xmlSTAT[i]}" = "name" -a "${xmlDATA[i]}" = "$Model" ] && [ "$i" -lt "$xINDEX" ]
		do
			let "i++"
		done
		#------------------------------ We've found our model --------------------------------------------------------

		while ! [ "${xmlSTAT[i]}" = ">" -a "${xmlDATA[i]}" = "model" ] && [ "$i" -lt "$xINDEX" ]
		do
			while ! [ "${xmlSTAT[i]}" = "<" -a "${xmlDATA[i]}" = "os" ] && [ "$i" -lt "$xINDEX" ]
			do
				let "i++"
			done
			#------------- Here start attributes of some os ------------------------------------------------------
			let "i++"
			ancor=$i
			while ! [ "${xmlSTAT[i]}" = "name" ] && [ "$i" -lt "$xINDEX" ]
			do
				let "i++"
			done
			if [ "${xmlSTAT[i]}" = "name" -a "${xmlDATA[i]}" != "$OS_short" ]; then
				continue
			fi

			i=$ancor
			while [ "${xmlSTAT[i]}" != "fname" -a "$i" -lt "$xINDEX" -a "${xmlSTAT[i]}" != ">" -a "${xmlDATA[i]}" != "os" ]	
			do
				let "i++"
			done
			if [ "$i" -lt "$xINDEX" -a "${xmlDATA[i]}" != "os" ]; then
				OS=${xmlDATA[i]}
			fi

			i=$ancor
			while [ "${xmlSTAT[i]}" != "file" -a "$i" -lt "$xINDEX" -a "${xmlSTAT[i]}" != ">" -a "${xmlDATA[i]}" != "os" ]	
			do
				let "i++"
			done
			if [ "$i" -lt "$xINDEX" -a "${xmlDATA[i]}" != "os" ]; then
				ImageFile=${xmlDATA[i]}
			else
				echo "Error: no image file name in $vendorXML. Abort..."
				echo `date +%F' '%T`" Error: no image file name in $vendorXML. Abort..." >> $logfile
				cleanup
				exit 1
			fi

			i=$ancor
			while [ "${xmlSTAT[i]}" != "md5" -a "$i" -lt "$xINDEX" -a "${xmlSTAT[i]}" != ">" -a "${xmlDATA[i]}" != "os" ]	
			do
				let "i++"
			done
			if [ "$i" -lt "$xINDEX" -a "${xmlDATA[i]}" != "os" ]; then
				MD5File=${xmlDATA[i]}
				DoMD5Check=0
			else
				echo "Warning: no md5 file name in $vendorXML."
				echo `date +%F' '%T`" Warning: no md5 file name in $vendorXML." >> $logfile
				DoMD5Check=1
			fi

			i=$ancor
			while [ "${xmlSTAT[i]}" != "xml" -a "$i" -lt "$xINDEX" -a "${xmlSTAT[i]}" != ">" -a "${xmlDATA[i]}" != "os" ]	
			do
				let "i++"
			done
			if [ "$i" -lt "$xINDEX" -a "${xmlDATA[i]}" != "os" ]; then
				DescriptorXML=${xmlDATA[i]}
				IsClone=0
			else
				IsClone=1
			fi

			break
		done
		#-------------------------------------------------------------------------------------------------------------

		echo "Model name: 			$Model"
		echo "OS: 				$OS"
		echo `date +%F' '%T`" Model name: 		$Model" >> $logfile
		echo `date +%F' '%T`" OS: 			$OS" >> $logfile

		#grab servers' url's
		declare -a Servers
		declare -i NumberOfServers=0
		i=0
		while ! [ "${xmlSTAT[i]}" = "<" -a "${xmlDATA[i]}" = "images" ] && [ "$i" -lt "$xINDEX" ]
		do
			let "i++"
		done
		while ! [ "${xmlSTAT[i]}" = "<" -a "${xmlDATA[i]}" = "server" ] && [ "$i" -lt "$xINDEX" ]
		do
			let "i++"
		done
		while [ "${xmlDATA[i]}" != "models" -a "$i" -lt "$xINDEX" ]
		do
			if [ "${xmlDATA[i]}" = "server" -a "${xmlSTAT[$((i+1))]}" = "url" ]; then
				Servers[NumberOfServers]="${xmlDATA[$((i+1))]}"
				let "NumberOfServers++"
			fi
			let "i++"
		done
	else
		echo "$vendorXML is missing. Abort..."
		echo `date +%F' '%T`" $vendorXML is missing. Abort..." >> $logfile
		cleanup
		exit 1
	fi


	#eval free memory
	freemem=`cat /proc/meminfo | grep MemFree`
	freemem=${freemem%' '*}
	freemem=${freemem##*' '}

	#mount free RAM as /tmp
	mount -t ramfs -o maxsize="$freemem""k" none /tmp
	if [ "$?" -eq 0 ]; then
		echo "Allocated $freemem k in ram for /tmp"
		echo `date +%F' '%T`" Allocated $freemem k in ram for /tmp" >> $logfile
	else
		echo "Error allocating $freemem k in ram for /tmp"
		echo `date +%F' '%T`" Error allocating $freemem k in ram for /tmp" >> $logfile
		cleanup
		exit 1
	fi
	cd /tmp

	#save platform.xml
	if [ -f "$platformXML" ]; then
		cp $platformXML /tmp/platform.xml
	fi

	#if there is a clone image
	if [ "$IsClone" -eq 0 ]; then
		echo "Descriptor file to download:	$DescriptorXML"
		echo
		echo `date +%F' '%T`" Descriptor file to download:	$DescriptorXML" >> $logfile
		# remove existing descriptor file
		[ -f "$DescriptorXML" ] && rm "$DescriptorXML"
		#try to download descriptor file
		DownloadedDescriptor=1;
		for (( i=0; i<$NumberOfServers; i++ ));
		do
			if [ "$DownloadedDescriptor" -ne 0 ] ; then
				echo "Trying to download descriptor file from ${Servers[i]}"
				echo -n `date +%F' '%T`" Trying to download descriptor file from ${Servers[i]}..." >> $logfile
				/mnt/launcher.sh "wget ${Servers[i]}$DescriptorXML"
				DownloadedDescriptor=$?
				if [ "$DownloadedDescriptor" -eq 27 ]; then
					echo -e "\n"`date +%F' '%T`" Download cancelled. Starting g4l." >> $logfile
					cleanup g4l 
				elif [ "$DownloadedDescriptor" -eq 0 ]; then
					echo "OK"
					echo "OK" >> $logfile
				else
					echo "failed"
					echo "failed" >> $logfile
				fi
			fi
		done
		if [ "$DownloadedDescriptor" -eq 1 ] ; then
			echo "Error: couldn't download descriptor file $DescriptorXML. Abort..."
			echo `date +%F' '%T`" Error: couldn't download descriptor file $DescriptorXML. Abort..." >> $logfile
			cleanup
			exit 1
		fi
		#clear parser data
		xmlSTAT=()			
		xmlDATA=()
		xINDEX=0				
		checkpoint=0				
		parserLINE=""			
		parserFLAG="ENABLED"		
		parserFILE=""			
		parserBUFF=""	
		parse "$DescriptorXML"
		#grab type number
		i=0
		while ! [ "${xmlSTAT[i]}" = "<" -a "${xmlDATA[i]}" = "type" -a "${xmlSTAT[$((i+1))]}" = "val" ] && [ "$i" -lt "$xINDEX" ]
		do
			let "i++"
		done
		if [ "$i" -lt "$xINDEX" ]; then
			Type=${xmlDATA[$((i+1))]}
		else
			echo "Error: no type in $DescriptorXML. Abort..."
			echo `date +%F' '%T`" Error: no type in $DescriptorXML. Abort..." >> $logfile
			cleanup
			exit 1
		fi
		#grab partiotion number
		i=0
		while ! [ "${xmlSTAT[i]}" = "<" -a "${xmlDATA[i]}" = "partition" -a "${xmlSTAT[$((i+1))]}" = "val" ] && [ "$i" -lt "$xINDEX" ]
		do
			let "i++"
		done
		if [ "$i" -lt "$xINDEX" ]; then
			PartitionNumber=${xmlDATA[$((i+1))]}
		else
			echo "Error: no partition number in $DescriptorXML. Abort..."
			echo `date +%F' '%T`" Error: no partition number in $DescriptorXML. Abort..." >> $logfile
			cleanup
			exit 1
		fi
		#grab image file
		i=0
		while ! [ "${xmlSTAT[i]}" = "<" -a "${xmlDATA[i]}" = "filename" -a "${xmlSTAT[$((i+1))]}" = "val" ] && [ "$i" -lt "$xINDEX" ]
		do
			let "i++"
		done
		if [ "$i" -lt "$xINDEX" ]; then
			ImageFile=${xmlDATA[$((i+1))]}
		else
			echo "Error: no image file name in $DescriptorXML. Abort..."
			echo `date +%F' '%T`" Error: no image file name number in $DescriptorXML. Abort..." >> $logfile
			cleanup
			exit 1
		fi
		Partition="/dev/"`cat /proc/partitions | grep '[sh]d[abcdefgh]' | head -n 1 |awk '{print $4}'`"$PartitionNumber"
		echo "Image file to download: 	$ImageFile"
		echo "Partition to restore:		$Partition"
		echo "Type:		 		$Type"
		echo	
		echo `date +%F' '%T`" Image file to download: 	$ImageFile" >> $logfile
		echo `date +%F' '%T`" Partition to restore:	$Partition" >> $logfile
		echo `date +%F' '%T`" Type:		 	$Type" >> $logfile
		# remove existing image file
		[ -f "$ImageFile" ] && rm "$ImageFile"
		#try to download image file
		ImageFileExists=1;
		for (( i=0; i<$NumberOfServers; i++ ));
		do
			if [ "$ImageFileExists" -ne 0 ] ; then
				echo "Check presence of $ImageFile on ${Servers[i]}"
				echo -n `date +%F' '%T`" Check presence of $ImageFile on ${Servers[i]}..." >> $logfile
				/mnt/launcher.sh "wget --spider ${Servers[i]}$ImageFile"
				ImageFileExists=$?
				if [ "$ImageFileExists" -eq 27 ]; then
					echo -e "\n"`date +%F' '%T`" Check presence of $ImageFile cancelled. Starting g4l." >> $logfile
					cleanup g4l
				elif [ "$ImageFileExists" -eq 0 ]; then
					echo "OK"
					echo "OK" >> $logfile
					Server=${Servers[i]}
				else
					echo "failed"
					echo "failed" >> $logfile
				fi
			fi
		done
		if [ "$ImageFileExists" -eq 1 ] ; then
			echo "Error: there is no image file $ImageFile. Abort..."
			echo `date +%F' '%T`" Error: there is no image file $ImageFile. Abort..." >> $logfile
			cleanup
			exit 1
		fi
		#restoring
		umount /mnt
  		case "$Type" in
 	  	"ntfsclone"	) wget -q -O - "$Server""$ImageFile" | gunzip --to-stdout - | ntfsclone --restore-image --overwrite "$Partition" - | tee -a $logfile
 	 			  if [ "$?" -ne ]; then
 	 				echo "Error: ntfsclone failed. Abort..."
 	 				echo `date +%F' '%T`" Error: ntfsclone failed. Abort..." >> $logfile
 	 				cleanup
 	 				exit 1
 	 			  fi ;;
 	  	"dd"		) wget -q -O - "$Server""$ImageFile" | gunzip --to-stdout - | dd of="$Partition" conv=notrunc | tee -a $logfile
 	 			  if [ "$?" -ne ]; then
 	 				echo "Error: dd failed. Abort..."
 	 				echo `date +%F' '%T`" Error: dd failed. Abort..." >> $logfile
 	 				cleanup
 	 				exit 1
 				  fi ;;
 	  	esac
	else
		echo "Image file to download: 	$ImageFile"
		echo "MD5 sum file to download:	$MD5File"
		echo
		echo `date +%F' '%T`" Image file to download: 	$ImageFile" >> $logfile
		echo `date +%F' '%T`" MD5 sum file to download:	$MD5File" >> $log
		# remove existing files
		[ -f $ImageFile ] && rm $ImageFile
		[ -f $MD5File ] && rm $MD5File
		#try to download files
		DownloadedImage=1;
		if [ "$DoMD5Check" -eq 0 ]; then
			DownloadedMD5=1
		else
			DownloadedMD5=0
		fi
		for (( i=0; i<$NumberOfServers; i++ ));
		do
			if [ "$DownloadedMD5" -ne 0 ] ; then
				echo "Trying to download md5 file from ${Servers[i]}"
				echo -n `date +%F' '%T`" Trying to download md5 file from ${Servers[i]}..." >> $logfile
				/mnt/launcher.sh "wget ${Servers[i]}$MD5File"
				DownloadedMD5=$?
				if [ "$DownloadedMD5" -eq 27 ]; then
					echo -e "\n"`date +%F' '%T`" Download cancelled. Starting g4l." >> $logfile
					cleanup g4l
				elif [ "$DownloadedMD5" -eq 0 ]; then
					echo "OK"
					echo "OK" >> $logfile
				else
					echo "failed"
					echo "failed" >> $logfile
				fi
			fi
			if [ "$DownloadedImage" -ne 0 -a "$DownloadedMD5" -eq 0 ]; then
				echo "Trying to download image file from ${Servers[i]}"
				echo -n `date +%F' '%T`" Trying to download image file from ${Servers[i]}..." >> $logfile
				/mnt/launcher.sh "wget ${Servers[i]}$ImageFile"
				DownloadedImage=$?
				if [ "$DownloadedImage" -eq 27 ]; then
					echo -e "\n"`date +%F' '%T`" Download cancelled. Starting g4l." >> $logfile
					cleanup g4l
				elif [ "$DownloadedImage" -eq 0 ]; then
					echo "OK"
					echo "OK" >> $logfile
				else
					echo "failed"
					echo "failed" >> $logfile
				fi
			fi
		done
		if [ "$DownloadedImage" -eq 1 ] ; then
			echo "Error: couldn't download image file $ImageFile. Abort..."
			echo `date +%F' '%T`" Error: couldn't download image file $ImageFile. Abort..." >> $logfile
			cleanup
			exit 1
		fi
		if [ "$DownloadedMD5" -eq 1 ] ; then
			echo "Error: couldn't download MD5 file $ImageMD5. Abort..."
			echo `date +%F' '%T`" Error: couldn't download MD5 file $ImageMD5. Abort..." >> $logfile
			cleanup
			exit 1
		fi
		#check md5 sum
		if [ "$DoMD5Check" -eq 0 ]; then
			MD5_got=`md5sum $ImageFile`
			MD5_got=${MD5_got%%$whiteTAG*}
			read MD5_etalon < $MD5File
			MD5_etalon=${MD5_etalon%%$whiteTAG*}
			if [ "$MD5_got" != "$MD5_etalon" ]; then
				echo "Error: MD5 sum check failed. Abort..."
				echo `date +%F' '%T`" Error: MD5 sum check failed. Abort..." >> $logfile
				cleanup
				exit 1
			else
				echo "MD5 sum check succeed."
				echo `date +%F' '%T`" MD5 sum check succeed." >> $logfile
			fi
		fi
		mnt_dir="/mnt"
		#unzip image file
		touch "/tmp/A"
		echo "A" > "/tmp/A"
		unzip "/tmp/$ImageFile" -d "$mnt_dir" < "/tmp/A" | tee -a $logfile
		if [ "$?" -ne 0 ]; then
			rm "/tmp/A"
			echo "Error unzipping $ImageFile to $mnt_dir. Abort..."
			echo `date +%F' '%T`" Error unzipping $ImageFile to $mnt_dir. Abort..." >> $logfile
			cleanup
			exit 1
		fi
		rm "/tmp/A"
	fi

	#restore platform.xml and menu.lst
	if [ -f "/tmp/platform.xml" ]; then
		cp -f /tmp/platform.xml $platformXML
 		case "$OS2" in
 		""	) ;;
 		"WinCE"	) cp -f "/mnt/menu.ce" "/mnt/menu.lst" ;;
 		"Linux"	) cp -f "/mnt/menu.lx" "/mnt/menu.lst" ;;
 		"WinXPe") cp -f "/mnt/menu.xp" "/mnt/menu.lst" ;;
 		esac
	fi

	echo "Update completed successfully."
	echo `date +%F' '%T`" Update completed successfully." >> $logfile

	cleanup

